#
# Great Doctor Mod
# CvGDEventManager
# 

from CvPythonExtensions import *

import Popup as PyPopup

import CvUtil

import CvEventManager
import sys
import PyHelpers
import CvMainInterface
import CvConfigParser
import CvGameInterface

# globals
###################################################

#######SD Tool Kit#######

import SdToolKit
sdEcho         = SdToolKit.sdEcho
sdModInit      = SdToolKit.sdModInit
sdModLoad      = SdToolKit.sdModLoad
sdModSave      = SdToolKit.sdModSave
sdEntityInit   = SdToolKit.sdEntityInit
sdEntityExists = SdToolKit.sdEntityExists
sdGetVal       = SdToolKit.sdGetVal
sdSetVal       = SdToolKit.sdSetVal

############################
			
gc = CyGlobalContext()	

PyPlayer = PyHelpers.PyPlayer
PyCity = PyHelpers.PyCity
PyGame = PyHelpers.PyGame()
localText = CyTranslator()

PRIMARY_TRAIT = "PRIMARY_TRAIT"
LAST_RATE = "LAST_RATE"
# Change this to true if at least one great person point should be required to
# enable trickling the great person point(s) related to the civilization 
# leaders trait.
# Default value is false
g_bRequireGreatPersonPoints = False

# Change this to true to enable the GP trickling in all player cities instead 
# of just player's capital city.
# Default value is true
g_bCapitalCitiesOnly = True

g_traitGreatPersonHashMap = {
								"TRAIT_PHILOSOPHICAL" : ["UNIT_SCIENTIST",1],
								"TRAIT_AGGRESSIVE" : ["UNIT_MERCHANT",1],
								"TRAIT_SPIRITUAL" : ["UNIT_PROPHET",1],
								"TRAIT_EXPANSIVE" : ["UNIT_ARTIST",1],
								"TRAIT_INDUSTRIOUS" : ["UNIT_ENGINEER",1],
								"TRAIT_CREATIVE" : ["UNIT_ARTIST",1],
								"TRAIT_FINANCIAL" : ["UNIT_MERCHANT",1],
								"TRAIT_ORGANIZED" : ["UNIT_ENGINEER",1]
							}
							
# Set to true to print out debug messages in the logs
g_bDebug = false

LOG_ONLY = 1
SCREEN_ONLY = 2
LOG_AND_SCREEN = 3

# Change to redirect the debug messages to the logs, screen or both.
g_iDebugMessageTarget = LOG_ONLY


class CvGPTrickleEventManager:
	
	def __init__(self, eventManager):
		
		eventManager.addEventHandler("cityDoTurn", self.onCityDoTurn)
		eventManager.addEventHandler("GameStart", self.onGameStart)
		eventManager.addEventHandler("OnLoad", self.onLoadGame)
		eventManager.addEventHandler("gameUpdate", self.onGameUpdate)
		eventManager.addEventHandler("cityBuilt", self.onCityBuilt)
		eventManager.addEventHandler("windowActivation", self.onWindowActivation)
					
	def onWindowActivation(self, argsList):
		'Called when the game window activates or deactivates'

		self.loadConfigFileData()


	def onGameUpdate(self, argsList):
		'Called every frame'

		# Get the selected city
		objCity = CyInterface().getHeadSelectedCity()

		# Return immediately if no city is currently selected
		if(objCity == None):
			return 
		
		# Return immediately if no city is currently selected
		if(objCity.isNone()):
			return 
							
		if (CyInterface().isCityScreenUp() and objCity.getOwner() == gc.getGame().getActivePlayer() ):
			if(self.fixTraitGreatPersonRate(objCity) != 0):
					CyInterface().setDirty(InterfaceDirtyBits.CityScreen_DIRTY_BIT, True)

		return 0


	def fixTraitGreatPersonRate(self, objCity):

		# Return immediately if the city passed in is invalid
		if(objCity == None):
			return false

		# Return immediately if the city passed in is invalid
		if(objCity.isNone()):
			return false
			
		# Return immediately if the mod is configured to only GP trickle in
		# capital cities and the city passed in is not a capital city.
		if(g_bCapitalCitiesOnly and not objCity.isCapital()):
			return false

		# Get the owner of the city
		objPlayer = gc.getPlayer(objCity.getOwner())

		# Get the primary trait ID for the player
		iPrimaryTrait = sdGetVal("GPTraitMod", objPlayer.getID(), PRIMARY_TRAIT)

		# Get the primary trait for the player
		objTraitInfo = gc.getTraitInfo(iPrimaryTrait)

		# Get the base great person point rate
		iBaseRate = self.calculateBaseTraitGreatPersonPoints(objCity, iPrimaryTrait)

		# Get the number of extra great person points that could be given each 
		# turn
		iExtraGreatPersonPoints = g_traitGreatPersonHashMap[objTraitInfo.getType()][1]

		# Get the last target rate
		iLastRate = sdGetVal("GPTraitMod", objPlayer.getID(), LAST_RATE)

		# If great person points are not required then give the city the extra 
		# great person points and return if the current base rate is zero.
		if(not g_bRequireGreatPersonPoints):

			if(iLastRate == 0):
				iBaseRate += iExtraGreatPersonPoints
				objCity.changeBaseGreatPeopleRate(iExtraGreatPersonPoints)
			else:
				if(CyInterface().isCityScreenUp()):
					return 0
					
				return iExtraGreatPersonPoints

			# Save the new target rate as the last rate
			sdSetVal("GPTraitMod", objPlayer.getID(), LAST_RATE, iBaseRate)

			return iExtraGreatPersonPoints
		
		# Set the base rate as the new target rate
		iTargetRate = iBaseRate
		
		# return false if the last target rate is equal to the new target rate
		if(iTargetRate == iLastRate):
			return false

		if(iLastRate == 0):
			iTargetRate+=iExtraGreatPersonPoints

		if(iTargetRate == 0 and iLastRate != 0):
			iBaseRate+=iExtraGreatPersonPoints

		iChangeRate = iTargetRate - iBaseRate

		# Debug code - start
		if(g_bDebug):
			self.printDebugMessage("Trait %s count %s %s" %(objTraitInfo.getType(), iExtraGreatPersonPoints, self.getTraitGreatPersonPointSourceCount(objCity, iPrimaryTrait)))
		# Debug code - end

		# Give or take away from the city the extra great person points
		objCity.changeBaseGreatPeopleRate(iChangeRate)

		# Save the new target rate as the last rate
		sdSetVal("GPTraitMod", objPlayer.getID(), LAST_RATE, iTargetRate)

		return iChangeRate
		

	# Calculates and returns the number of great person points produced by the 
	# city passed in for the trait passed in.
	def calculateBaseTraitGreatPersonPoints(self, objCity, iTrait):

		objSpecialistInfo = None
		iSpecialistInfo = -1

		# Return 0 immediately if the city passed in is invalid
		if(objCity == None):
			return
			
		# Return 0 immediately if the trait passed in is invalid
		if(iTrait < 0):
			return 0

		objTraitInfo = gc.getTraitInfo(iTrait)
		
		# Return 0 immediately if the trait passed in is invalid
		if(objTraitInfo == None):
			return 0		
		# Get the unit type ID number
		iUnitType = gc.getInfoTypeForString(g_traitGreatPersonHashMap[objTraitInfo.getType()][0])

		# Get the unit info
		objUnitInfo = gc.getUnitInfo(iUnitType)

		# Get the unit class type ID
		iUnitClassType = objUnitInfo.getUnitClassType()

		# Get the owner of the city passed in
		objPlayer = gc.getPlayer(objCity.getOwner())
		
		# Go through the specialist infos loaded in the game and find the one 
		# that has the great people unit class that matches the unit class type
		for iSpecialistInfo in range(gc.getNumSpecialistInfos()):

			# Get the specialist into
			objSpecialistInfo = gc.getSpecialistInfo(iSpecialistInfo)
			
			# Break out of the loop if the great people unit class in the 
			# specialist matches the unit class type
			if(objSpecialistInfo.getGreatPeopleUnitClass() == iUnitClassType):
				break
		
		# Count the number of specialists and free specialists in the city and 
		# multiply it by the rate change to get the number of great person 
		# points
		iGreatPersonPoints = objSpecialistInfo.getGreatPeopleRateChange() * (objCity.getSpecialistCount(iSpecialistInfo) + objCity.getFreeSpecialistCount(iSpecialistInfo))

		# Get the building list from the city passed in.
		buildingList = PyCity(objPlayer.getID(), objCity.getID()).getBuildingList()

		# Go through each of the buildings in the city and add the great person 
		# points that produce great person points for the unit class type
		for iBuildingInfo in buildingList:
			# Get the building info
			objBuildingInfo = gc.getBuildingInfo(iBuildingInfo)

			# if the building produces the great person points for the unit 
			# class type then add them to the running total.
			if(objBuildingInfo.getGreatPeopleUnitClass() == iUnitClassType):			
				iGreatPersonPoints += objBuildingInfo.getGreatPeopleRateChange()

		# Debug code - start
		if(g_bDebug):
			self.printDebugMessage("%s GPPs for the %s trait in %s" %(iGreatPersonPoints, objTraitInfo.getType(), objCity.getName()))
		# Debug code - end

		return iGreatPersonPoints
		
	
	# Returns the number of sources producing great person points for the trait
	# passed in.
	def getTraitGreatPersonPointSourceCount(self, objCity, iTrait):

		objSpecialistInfo = None
		iSpecialistInfo = -1

		# Return 0 immediately if the city passed in is invalid
		if(objCity == None):
			return
			
		# Return 0 immediately if the trait passed in is invalid
		if(iTrait < 0):
			return 0

		objTraitInfo = gc.getTraitInfo(iTrait)
		
		# Return 0 immediately if the trait passed in is invalid
		if(objTraitInfo == None):
			return 0
		
		# Get the unit type ID number
		iUnitType = gc.getInfoTypeForString(g_traitGreatPersonHashMap[objTraitInfo.getType()][0])

		# Get the unit info
		objUnitInfo = gc.getUnitInfo(iUnitType)

		# Get the unit class type ID
		iUnitClassType = objUnitInfo.getUnitClassType()

		# Get the owner of the city passed in
		objPlayer = gc.getPlayer(objCity.getOwner())
		
		# Go through the specialist infos loaded in the game and find the one 
		# that has the great people unit class that matches the unit class type
		for iSpecialistInfo in range(gc.getNumSpecialistInfos()):

			# Get the specialist into
			objSpecialistInfo = gc.getSpecialistInfo(iSpecialistInfo)
			
			# Break out of the loop if the great people unit class in the 
			# specialist matches the unit class type
			if(objSpecialistInfo.getGreatPeopleUnitClass() == iUnitClassType):
				break
		
		# Count the number of specialists and free specialists in the city
		iGreatPersonPointSourceCount = objCity.getSpecialistCount(iSpecialistInfo) + objCity.getFreeSpecialistCount(iSpecialistInfo)

		# Get the building list from the city passed in.
		buildingList = PyCity(objPlayer.getID(), objCity.getID()).getBuildingList()

		# Go through each of the buildings in the city and count the ones that
		# produce great person points for the unit class type
		for iBuildingInfo in buildingList:

			# Get the building info
			objBuildingInfo = gc.getBuildingInfo(iBuildingInfo)

			# If the building is producing great people points that match the 
			# unit class type then count the building as one of the sources
			if(objBuildingInfo.getGreatPeopleUnitClass() == iUnitClassType):			
				iGreatPersonPointSourceCount += 1

		# Debug code - start
		if(g_bDebug):
			self.printDebugMessage("%s sources of GPP for the %s trait in %s" %(iGreatPersonPointSourceCount, objTraitInfo.getType(), objCity.getName()))
		# Debug code - end

		return iGreatPersonPointSourceCount
	
	
	# Called when a city is built. Will fix-up the city's great person points.
	def onCityBuilt(self, argsList):
		'City Built'
		objCity = argsList[0]

		# Return immediately if the city passed in is invalid
		if(objCity == None):
			return
			
		# Return immediately if the mod is configured to only GP trickle in
		# capital cities and the city passed in is not a capital city.
		if(g_bCapitalCitiesOnly and not objCity.isCapital()):
			return
		
		# Fix the cities great person rate depending on its owner's primary 
		# trait
		self.fixTraitGreatPersonRate(objCity)


	# Called at the end of each cities upkeep turn. Will fix-up the city's 
	# great person points.
	def onCityDoTurn(self, argsList):
		'City Production'

		objCity = argsList[0]
		iPlayer = argsList[1]

		# Return immediately if the city passed in is invalid
		if(objCity == None):
			return
			
		# Return immediately if the mod is configured to only GP trickle in
		# capital cities and the city passed in is not a capital city.
		if(g_bCapitalCitiesOnly and not objCity.isCapital()):
			return
		
		# Fix the cities great person rate depending on its owner's primary 
		# trait
		iProgressPoints = self.fixTraitGreatPersonRate(objCity)
		
		# Get the primary trait ID for the player
		iPrimaryTrait = sdGetVal("GPTraitMod", objCity.getOwner(), PRIMARY_TRAIT)

		# Get the primary trait for the player
		objTraitInfo = gc.getTraitInfo(iPrimaryTrait)

		# Get the unit type ID number
		iUnitType = gc.getInfoTypeForString(g_traitGreatPersonHashMap[objTraitInfo.getType()][0])
		
		objCity.changeGreatPeopleUnitProgress(iUnitType, iProgressPoints)


	# Loads the configuration file if it exists.
	def loadConfigFileData(self):
	
		# Try to load the file
		config = CvConfigParser.CvConfigParser("GP Trickle Mod Config.ini")

		# If the file was successfully loaded then initialize the configurable variables
		if(config != None):
			global g_traitGreatPersonHashMap
			global g_bRequireGreatPersonPoints
			global g_bCapitalCitiesOnly
			
			g_bRequireGreatPersonPoints = config.getboolean("GP Trickle Mod", "Require Great Person Points", false)
			g_bCapitalCitiesOnly = config.getboolean("GP Trickle Mod", "Capital Cities Only", true)
			
			# Go through all of the traits that have been loaded into the game
			# and check to see if they were customized through the 
			# configuration file
			for i in range(gc.getNumTraitInfos()):
			
				# Get the string representation of the trait type
				strTraitType = gc.getTraitInfo(i).getType()
				
				# Try to get the great person that the trait maps to.
				strGreatPerson = config.get("GP Trickle Mod", strTraitType, g_traitGreatPersonHashMap[strTraitType][0])

				# Get the number of points that the great person string maps to.
				iGreatPersonPoints = config.getint("GP Trickle Mod", strGreatPerson, 1)

				g_traitGreatPersonHashMap[strTraitType] = [strGreatPerson, iGreatPersonPoints]
				
				# Debug code - start
				if(g_bDebug):
					self.printDebugMessage("Loaded %s mapped to %s for %s points" %(strTraitType, strGreatPerson, iGreatPersonPoints))
				# Debug code - end


	# Called when a game is loaded and loads the configuration file.
	def onLoadGame(self, argsList):
		self.loadConfigFileData()
		
		return 0


	# Called when the game starts, sets up the initial data structures to hold 
	# the GP trickle data and loads the configuration file.
	def onGameStart(self, argsList):
		'Called at the start of the game'

		for iPlayer in range(gc.getMAX_PLAYERS()):
			if (gc.getPlayer(iPlayer).isAlive() and not gc.getPlayer(iPlayer).isBarbarian()):
				self.setupGPTraitData(iPlayer)					

		self.loadConfigFileData()
		

	# Sets up the great person trickle trait data. This method should be called
	# for each player in the game human or not.
	def setupGPTraitData(self, iPlayer):
	
		# Return immediately if the player ID passed in is invalid
		if(iPlayer < 0):
			return
			
		# Get the PyPlayer instance
		objPlayer = PyPlayer(iPlayer)
	
		# Get the player's trait list
		playerTraitList = objPlayer.getTraitList()
		
		# Randomly get the primary trait from the list of player traits
		iPrimaryTrait = playerTraitList[gc.getGame().getMapRand().get(len(playerTraitList)-1,"Leader Trait")]

		# Setup the trait data hash map
		gpTraitDataHash = { PRIMARY_TRAIT : iPrimaryTrait,
							LAST_RATE : 0 }

		# Debug code - start
		if(g_bDebug):
			self.printDebugMessage("%s primary trait is %s" %(gc.getPlayer(iPlayer).getName(),gc.getTraitInfo(iPrimaryTrait).getDescription()))
		# Debug code - end
	
		sdEntityInit("GPTraitMod", iPlayer, gpTraitDataHash)


	# Prints the debug message to the logs, screen or both	
	def printDebugMessage(self, strMessage):
	
		if(g_iDebugMessageTarget == LOG_ONLY):
			CvUtil.pyPrint(strMessage)
		elif(g_iDebugMessageTarget == SCREEN_ONLY):
			CyInterface().addImmediateMessage(strMessage,"")
		else:
			CvUtil.pyPrint(strMessage)
			CyInterface().addImmediateMessage(strMessage,"")


